@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -26,8 +26,12 @@ import sys
 import time
 import zlib
 import base64
+import inspect
+import warnings
 import traceback
+import contextlib
 from random import Random
+from unittest import TestCase
 
 import attr
 from coverage import CoverageData
@@ -38,17 +42,19 @@ import hypothesis.strategies as st
 from hypothesis import __version__
 from hypothesis.errors import Flaky, Timeout, NoSuchExample, \
     Unsatisfiable, DidNotReproduce, InvalidArgument, DeadlineExceeded, \
-    MultipleFailures, FailedHealthCheck, UnsatisfiedAssumption, \
-    HypothesisDeprecationWarning
+    MultipleFailures, FailedHealthCheck, HypothesisWarning, \
+    UnsatisfiedAssumption, HypothesisDeprecationWarning
 from hypothesis.control import BuildContext
-from hypothesis._settings import settings as Settings
 from hypothesis._settings import Phase, Verbosity, HealthCheck, \
-    PrintSettings, note_deprecation
+    PrintSettings
+from hypothesis._settings import settings as Settings
+from hypothesis._settings import note_deprecation
 from hypothesis.executors import new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
-from hypothesis.internal.compat import ceil, str_to_bytes, \
-    benchmark_time, get_type_hints, getfullargspec, encoded_filepath
+from hypothesis.internal.compat import ceil, hbytes, qualname, \
+    str_to_bytes, benchmark_time, get_type_hints, getfullargspec, \
+    encoded_filepath, bad_django_TestCase
 from hypothesis.internal.coverage import IN_COVERAGE_TESTS
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.escalation import is_hypothesis_file, \
@@ -113,7 +119,6 @@ def seed(seed):
     derandomize setting if it is present.
 
     """
-
     def accept(test):
         test._hypothesis_internal_use_seed = seed
         return test
@@ -133,7 +138,6 @@ def reproduce_failure(version, blob):
     in the event that you don't have access to the test database. Because of
     this, *no* compatibility guarantees are made between different versions of
     Hypothesis - its API may change arbitrarily from version to version.
-
     """
     def accept(test):
         test._hypothesis_internal_use_reproduce_failure = (version, blob)
@@ -349,7 +353,6 @@ def skip_exceptions_to_reraise():
 
     This is intended to cover most common test runners; if you would
     like another to be added please open an issue or pull request.
-
     """
     import unittest
     # This is a set because nose may simply re-export unittest.SkipTest
@@ -880,14 +883,38 @@ class StateForActualGivenExecution(object):
             report('Flaky example! ' + message)
 
 
+@contextlib.contextmanager
+def fake_subTest(self, msg=None, **__):
+    """Monkeypatch for `unittest.TestCase.subTest` during `@given`.
+
+    If we don't patch this out, each failing example is reported as a
+    seperate failing test by the unittest test runner, which is
+    obviously incorrect. We therefore replace it for the duration with
+    this version.
+    """
+    warnings.warn(
+        'subTest per-example reporting interacts badly with Hypothesis '
+        'trying hundreds of examples, so we disable it for the duration of '
+        'any test that uses `@given`.', HypothesisWarning, stacklevel=2
+    )
+    yield
+
+
 def given(*given_arguments, **given_kwargs):
     """A decorator for turning a test function that accepts arguments into a
     randomized test.
 
     This is the main entry point to Hypothesis.
-
     """
     def run_test_with_generator(test):
+        if hasattr(test, '_hypothesis_internal_test_function_without_warning'):
+            # Pull out the original test function to avoid the warning we
+            # stuck in about using @settings without @given.
+            test = test._hypothesis_internal_test_function_without_warning
+        if inspect.isclass(test):
+            # Provide a meaningful error to users, instead of exceptions from
+            # internals that assume we're dealing with a function.
+            raise InvalidArgument('@given cannot be applied to a class.')
         generator_arguments = tuple(given_arguments)
         generator_kwargs = dict(given_kwargs)
 
@@ -945,6 +972,22 @@ def given(*given_arguments, **given_kwargs):
             )
             arguments, kwargs, test_runner, search_strategy = processed_args
 
+            runner = getattr(search_strategy, 'runner', None)
+            if isinstance(runner, TestCase) and test.__name__ in dir(TestCase):
+                msg = ('You have applied @given to the method %s, which is '
+                       'used by the unittest runner but is not itself a test.'
+                       '  This is not useful in any way.' % test.__name__)
+                fail_health_check(settings, msg, HealthCheck.not_a_test_method)
+            if bad_django_TestCase(runner):  # pragma: no cover
+                # Covered by the Django tests, but not the pytest coverage task
+                raise InvalidArgument(
+                    'You have applied @given to a method on %s, but this '
+                    'class does not inherit from the supported versions in '
+                    '`hypothesis.extra.django`.  Use the Hypothesis variants '
+                    'to ensure that each example is run in a separate '
+                    'database transaction.' % qualname(type(runner))
+                )
+
             state = StateForActualGivenExecution(
                 test_runner, search_strategy, test, settings, random,
                 had_seed=wrapped_test._hypothesis_internal_use_seed
@@ -998,7 +1041,15 @@ def given(*given_arguments, **given_kwargs):
                 return
 
             try:
-                state.run()
+                if isinstance(runner, TestCase) and hasattr(runner, 'subTest'):
+                    subTest = runner.subTest
+                    try:
+                        setattr(runner, 'subTest', fake_subTest)
+                        state.run()
+                    finally:
+                        setattr(runner, 'subTest', subTest)
+                else:
+                    state.run()
             except BaseException:
                 generated_seed = \
                     wrapped_test._hypothesis_internal_use_generated_seed
@@ -1019,6 +1070,9 @@ def given(*given_arguments, **given_kwargs):
             if not (attrib.startswith('_') or hasattr(wrapped_test, attrib)):
                 setattr(wrapped_test, attrib, getattr(test, attrib))
         wrapped_test.is_hypothesis_test = True
+        if hasattr(test, '_hypothesis_internal_settings_applied'):
+            # Used to check if @settings is applied twice.
+            wrapped_test._hypothesis_internal_settings_applied = True
         wrapped_test._hypothesis_internal_use_seed = getattr(
             test, '_hypothesis_internal_use_seed', None
         )
@@ -1081,11 +1135,15 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
                     report(lambda: u'Found satisfying example %s' % (
                         nicerepr(result),
                     ))
-                else:
+                    last_data[0] = data
+                elif (
+                    sort_key(hbytes(data.buffer)) <
+                    sort_key(last_data[0].buffer)
+                ):
                     report(lambda: u'Shrunk example to %s' % (
                         nicerepr(result),
                     ))
-                last_data[0] = data
+                    last_data[0] = data
         if success and not data.frozen:
             data.mark_interesting()
     start = time.time()
